@@ -345,8 +345,8 @@ GST_START_TEST (test_pad_templates)
   test = g_object_new (gst_test_element_get_type (), NULL);
   test2 = g_object_new (gst_test_element2_get_type (), NULL);
 
-  g_object_unref (test);
-  g_object_unref (test2);
+  gst_object_unref (test);
+  gst_object_unref (test2);
 }
 
 GST_END_TEST;
@@ -654,7 +654,6 @@ GST_START_TEST (test_request_pad_templates)
   GHashTableIter iter;
   gpointer key, value;
   const gchar *pad_name, *templ_name;
-  GSList *padname_blacklists = NULL, *item;
   GError *err = NULL;
 
   padnames = g_hash_table_new (g_str_hash, g_str_equal);
@@ -703,23 +702,7 @@ GST_START_TEST (test_request_pad_templates)
   g_hash_table_insert (padnames, (gpointer) "src_%s", (gpointer) "src_%s");
   g_hash_table_insert (padnames, (gpointer) "src_%u_%s",
       (gpointer) "src_%u_%s");
-
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%u%u");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%u_%d");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%u_%u_");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%u_%s_%s");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%s_%u");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%s_%s");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%s_%s_%s");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%s_blah");
+  g_hash_table_insert (padnames, (gpointer) "src_foo_bar", (gpointer) "src_%s");
 
   test = g_object_new (gst_test_element3_get_type (), NULL);
 
@@ -729,7 +712,7 @@ GST_START_TEST (test_request_pad_templates)
     pad_name = (const gchar *) key;
     templ_name = (const gchar *) value;
 
-    pad = gst_element_get_request_pad (GST_ELEMENT (test), pad_name);
+    pad = gst_element_request_pad_simple (GST_ELEMENT (test), pad_name);
     fail_unless (pad != NULL);
     gst_element_release_request_pad (GST_ELEMENT (test), pad);
     gst_object_unref (pad);
@@ -742,16 +725,6 @@ GST_START_TEST (test_request_pad_templates)
     gst_object_unref (pad);
   }
 
-  item = padname_blacklists;
-
-  /* check invalid request pad name */
-  while (item) {
-    pad_name = (const gchar *) (item->data);
-    item = g_slist_next (item);
-    pad = gst_element_get_request_pad (GST_ELEMENT (test), pad_name);
-    fail_unless (pad == NULL);
-  }
-
   /* check it working with some APIs
    * gst_element_link/link_pads */
   sink = gst_element_factory_make ("fakesink", "sink");
@@ -763,8 +736,8 @@ GST_START_TEST (test_request_pad_templates)
           "sink"));
   gst_element_unlink (GST_ELEMENT (test), sink);
 
-  g_object_unref (test);
-  g_object_unref (sink);
+  gst_object_unref (test);
+  gst_object_unref (sink);
 
   /* gst_parse_launch */
   gst_plugin_register_static (GST_VERSION_MAJOR,
@@ -782,13 +755,163 @@ GST_START_TEST (test_request_pad_templates)
   if (err) {
     g_error_free (err);
   }
-  g_slist_free (padname_blacklists);
   g_hash_table_unref (padnames);
   gst_object_unref (pipeline);
 }
 
 GST_END_TEST;
 
+GST_START_TEST (test_forbidden_pad_template_names)
+{
+  const gchar *pad_name;
+  GSList *padname_blacklists = NULL, *item;
+
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%u%u");
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%u_%s_%s");
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%s_%u");
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%s_%s");
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%s_%s_%s");
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%s_blah");
+
+  item = padname_blacklists;
+
+  /* check invalid request pad name */
+  while (item) {
+    pad_name = (const gchar *) (item->data);
+    item = g_slist_next (item);
+    ASSERT_WARNING (gst_pad_template_new (pad_name, GST_PAD_SRC,
+            GST_PAD_REQUEST, GST_CAPS_ANY));
+  }
+
+  g_slist_free (padname_blacklists);
+}
+
+GST_END_TEST;
+
+static gboolean run_foreach_thread;
+
+/* thread function that just adds/removes pads while main thread iterates pads */
+static gpointer
+thread_add_remove_pads (GstElement * e)
+{
+  GPtrArray *pads;
+  guint n, c = 0;
+
+  pads = g_ptr_array_new ();
+
+  THREAD_START ();
+
+  while (g_atomic_int_get (&run_foreach_thread)) {
+    GstPad *p;
+    gchar name[16];
+
+    /* add a new pad */
+    g_snprintf (name, 16, "pad_%u", c++);
+    p = gst_pad_new (name, g_random_boolean ()? GST_PAD_SRC : GST_PAD_SINK);
+    g_ptr_array_add (pads, p);
+    gst_element_add_pad (e, p);
+
+    THREAD_SWITCH ();
+
+    /* and remove a random pad */
+    if (g_random_boolean () || pads->len > 100) {
+      n = g_random_int_range (0, pads->len);
+      p = g_ptr_array_remove_index (pads, n);
+      gst_element_remove_pad (e, p);
+    }
+
+    THREAD_SWITCH ();
+  }
+
+  g_ptr_array_free (pads, TRUE);
+  return NULL;
+}
+
+typedef struct
+{
+  GQuark q;
+  GstPadDirection dir;          /* GST_PAD_UNKNOWN = both are allowed */
+  gboolean func_called;
+} PadChecks;
+
+static gboolean
+pad_foreach_func (GstElement * e, GstPad * pad, gpointer user_data)
+{
+  PadChecks *checks = user_data;
+
+  /* check we haven't visited this pad already */
+  fail_if (g_object_get_qdata (G_OBJECT (pad), checks->q) != NULL);
+
+  g_object_set_qdata (G_OBJECT (pad), checks->q, GINT_TO_POINTER (1));
+
+  if (checks->dir != GST_PAD_UNKNOWN) {
+    fail_unless_equals_int (checks->dir, GST_PAD_DIRECTION (pad));
+  }
+  checks->func_called = TRUE;
+  return TRUE;
+}
+
+GST_START_TEST (test_foreach_pad)
+{
+  PadChecks checks = { 0, GST_PAD_UNKNOWN, FALSE };
+  GstElement *e;
+  gint i;
+
+  e = gst_bin_new ("testbin");
+
+  /* function should not be called if there are no pads! */
+  gst_element_foreach_pad (e, pad_foreach_func, &checks);
+  fail_if (checks.func_called);
+
+  g_atomic_int_set (&run_foreach_thread, TRUE);
+
+  MAIN_INIT ();
+  MAIN_START_THREAD_FUNCTION (0, thread_add_remove_pads, e);
+  MAIN_SYNCHRONIZE ();
+
+  for (i = 0; i < 1000; ++i) {
+    gchar num[32];
+
+    g_snprintf (num, 32, "foreach-test-%u", i);
+
+    checks.q = g_quark_from_string (num);
+    checks.func_called = FALSE;
+    if (g_random_boolean ()) {
+      checks.dir = GST_PAD_UNKNOWN;
+      gst_element_foreach_pad (e, pad_foreach_func, &checks);
+    } else if (g_random_boolean ()) {
+      checks.dir = GST_PAD_SRC;
+      gst_element_foreach_src_pad (e, pad_foreach_func, &checks);
+    } else {
+      checks.dir = GST_PAD_SINK;
+      gst_element_foreach_sink_pad (e, pad_foreach_func, &checks);
+    }
+
+    THREAD_SWITCH ();
+  }
+
+  g_atomic_int_set (&run_foreach_thread, FALSE);
+
+  MAIN_STOP_THREADS ();
+
+  /* function should be called if there are pads */
+  checks.q = g_quark_from_string ("fini");
+  checks.dir = GST_PAD_UNKNOWN;
+  checks.func_called = FALSE;
+  gst_element_foreach_pad (e, pad_foreach_func, &checks);
+  fail_if (e->numpads > 0 && !checks.func_called);
+
+  gst_object_unref (e);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_element_suite (void)
 {
@@ -804,6 +927,8 @@ gst_element_suite (void)
   tcase_add_test (tc_chain, test_pad_templates);
   tcase_add_test (tc_chain, test_property_notify_message);
   tcase_add_test (tc_chain, test_request_pad_templates);
+  tcase_add_test (tc_chain, test_forbidden_pad_template_names);
+  tcase_add_test (tc_chain, test_foreach_pad);
 
   return s;
 }
